/* Icon Plus
 * Copyright (C) 2001 Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: ic_os2.h,v 1.3 2001/05/29 05:28:20 glass Exp $
 */

#ifndef IC_OS2_H
#define IC_OS2_H

#include "ic_config.h"
#include "ic_file.h"
#include "ic_palette.h"
#include "ic_bitmap.h"

struct os2_bitmap_array_header
{
  UINT16 type;            /* Type of structure. */
  UINT32 cbsize;          /* Size of the structure in bytes. */
  UINT32 next;            /* Offset of the next bitmap array structure from
                             the start of the file. */
  UINT16 display_width;   /* Device width, in pels. */
  UINT16 display_height;  /* Device height, in pels. */
};

struct os2_bitmap_file_header
{
  UINT16 type;            /* Type of resource the file contains. */
  UINT32 cbsize;          /* Size of the structure in bytes. */
  INT16  xhotspot;        /* Width of hotspot for icons and pointers. */
  INT16  yhotspot;        /* Height of hotspot for icons and pointers. */
  UINT32 bitmap_offset;   /* Offset in bytes to beginning of bitmap pel
                             data in the file, from the start of the definition. */
};

struct os2_bitmap_header_v12 : public os2_bitmap_file_header
{
  UINT32 cbfix;           /* Size of the structure in bytes. */
  UINT16 width;           /* Bitmap width in pels. */
  UINT16 height;          /* Bitmap height in pels. */
  UINT16 planes;          /* Number of bit planes. */
  UINT16 bits_per_pixel;  /* Number of bits per pel within a plane. */

  ICPalette palette;      /* This is not a part of the standard OS/2 header! */
};

struct os2_bitmap_header_v20 : public os2_bitmap_file_header
{
  UINT32 cbfix;           /* Size of the structure in bytes. */
  UINT32 width;           /* Bitmap width in pels. */
  UINT32 height;          /* Bitmap height in pels. */
  UINT16 planes;          /* Number of bit planes. */
  UINT16 bits_per_pixel;  /* Number of bits per pel within a plane. */
  UINT32 compression;     /* Compression scheme used to store the bit map. */
  UINT32 bitmap_size;     /* Length of bitmap storage data, in bytes.
                             If the bit map is uncompressed, zero
                             (the default) can be specified for this. */
  UINT32 devXres;         /* Horizontal component of the resolution of target device. */
  UINT32 devYres;         /* Vertical component of the resolution of target device. */
  UINT32 clrUsed;         /* Number of color indexes used. */
  UINT32 clrImportant;    /* Minimum number of color indexes for satisfactory
                             appearance of the bit map. */
  UINT16 units;           /* Units of measure. */
  UINT16 reserved;        /* Reserved. */
  UINT16 recording;       /* Recording algorithm. */
  UINT16 rendering;       /* Halftoning algorithm. */
  UINT32 size1;           /* Size value 1. */
  UINT32 size2;           /* Size value 2. */
  UINT32 color_encoding;  /* Color encoding. */
  UINT32 app_identifier;  /* Reserved for application use. */

  ICPalette palette;      /* This is not a part of the standard OS/2 header! */
};

#define OS2_BFT_BITMAPARRAY   0x4142
#define OS2_BFT_BITMAP        0x4D42
#define OS2_BFT_ICON          0x4349
#define OS2_BFT_POINTER       0x4540
#define OS2_BFT_COLORICON     0x4943
#define OS2_BFT_COLORPOINTER  0x5043

struct os2_icon_info
{
  os2_bitmap_array_header header;
  os2_bitmap_header_v20   xorand;
  os2_bitmap_header_v20   bitmap;

  class ICIcon* icon;
};

class ICOS2
{
  public:
    virtual void load( ICFile* load_file  );
    virtual void save( ICFile* save_file, icon_type );

  protected:
    virtual os2_bitmap_array_header load_barray_header();
    virtual os2_bitmap_header_v20   load_bitmap_header();

    virtual void load_bitmap       ();
    virtual void load_bitmap_data  ( ICBitmap& );
    virtual void save_bitmap_data  ( ICBitmap& );
    virtual void save_barray_header( const os2_bitmap_array_header& i );
    virtual void save_bitmap_header( const os2_bitmap_header_v20&   i );

  private:
    ICFile* file;
};

#endif

